#ifndef ALIAS_H
#define ALIAS_H
#include "kwdset.h"

// aliases for type of solved problem
enum problemtype {linear_statics=1, eigen_dynamics=2, forced_dynamics=3,
                  linear_stability=5,mat_nonlinear_statics=10,geom_nonlinear_statics=11,
                  mech_timedependent_prob=15,growing_mech_structure=17,
                  earth_pressure=20,var_stiff_method=25,layered_linear_statics=30,
                  lin_floating_subdomain=35,nonlin_floating_subdomain=36,
                  hemivar_inequal=37,
                  nonlinear_dynamics=40,load_balancing=51};
const enumstr problemtypestr[] = {{"linear_statics",1}, {"eigen_dynamics",2}, {"forced_dynamics",3},
                                  {"linear_stability",5}, {"mat_nonlinear_statics",10}, {"geom_nonlinear_statics",11},
                                  {"mech_timedependent_prob",15}, {"growing_mech_structure",17}, {"earth_pressure",20},
                                  {"var_stiff_method",25}, {"layered_linear_statics",30}, {"lin_floating_subdomain",35},
                                  {"nonlin_floating_subdomain",36}, {"hemivar_inequal",37},
                                  {"nonlinear_dynamics",40},{"load_balancing",51}};
const kwdset problemtype_kwdset(sizeof(problemtypestr)/sizeof(*problemtypestr), problemtypestr);



// aliases for forced dynamics solvers
enum forcedsolver {newmark=1,findiff=2,modal_analysis=51};
const enumstr forcedsolverstr[] = {{"newmark",1}, {"findiff",2}, {"modal_analysis",51}};
const kwdset forcedsolver_kwdset(sizeof(forcedsolverstr)/sizeof(*forcedsolverstr), forcedsolverstr);



//aliases for nonlinear statics solvers
enum nonlinsolvertype {arcl=1,newton=2,arclrv1=3,newtonrv1=4,newtonrestart=5,displctrl=6,displctrlrv=7,arclrv=8,adaptram=10};
const enumstr nonlinsolvertypestr[] = {{"arcl",1}, {"newton",2}, {"arclrv1",3}, {"newtonrv1",4}, {"newtonrestart",5}, 
                                        {"displctrl",6}, {"displctrlrv",7}, {"arclrv",8}, {"adaptram",10}};
const kwdset nonlinsolvertype_kwdset(sizeof(nonlinsolvertypestr)/sizeof(*nonlinsolvertypestr), nonlinsolvertypestr);



//aliases for nonlinear time-dependent solvers
enum nonlintimesolvertype {fullnewton=1,modnewton=2};
const enumstr nonlintimesolvertypestr[] = {{"fullnewton",1},{"modnewton",2}};
const kwdset  nonlintimesolvertype_kwdset(sizeof(nonlintimesolvertypestr)/sizeof(*nonlintimesolvertypestr), nonlintimesolvertypestr);



// aliases for earth pressure solvers
enum epsolvertype {gep_sol=1, gepvarsup_sol=2, epplast_sol=3};
const enumstr epsolvertypestr[] = {{"gep_sol",1}, {"gepvarsup_sol",2}, {"epplast_sol",3}};
const kwdset epsolvertypes_kwdset(sizeof(epsolvertypestr)/sizeof(*epsolvertypestr), epsolvertypestr);



// aliases for type of displacement norm computation
enum displacementnorm {alldofs=1,seldofs=2,seldofscoord=3,selecnodes=6,nodesdistincr=8};
const enumstr displacementnormstr[] = {{"alldofs",1}, {"seldofs",2}, {"seldofscoord",3}, {"selnodes",6}, {"nodesdistincr",8}};
const kwdset displacementnorm_kwdset(sizeof(displacementnormstr)/sizeof(*displacementnormstr), displacementnormstr);



// aliases for types fo stress return algorithms
enum stressretalgtype {nostressretalg=0,cp=1,gsra=10};
const enumstr stressretalgtypestr[] = {{"nostressretalg",0}, {"cp",1}, {"gsra",10}};
const kwdset stressretalgtype_kwdset(sizeof(stressretalgtypestr)/sizeof(*stressretalgtypestr), stressretalgtypestr);


// aliases for types of stiffness matrix computation
enum stiffmatrix {initial_stiff=1,tangent_stiff=2,secant_stiff=3};
const enumstr stiffmatrixstr[] = {{"initial_stiff",1}, {"tangent_stiff",2}, {"secant_stiff",3}};
const kwdset stiffmatrix_kwdset(sizeof(stiffmatrixstr)/sizeof(*stiffmatrixstr), stiffmatrixstr);



// aliases for damping types
enum damping {nodamp=0,massdamp=1,stiffdamp=2,rayleighdamp=3};
const enumstr dampingstr[] = {{"nodamp",0}, {"massdamp",1}, {"stiffdamp",2}, {"rayleighdamp",3}};
const kwdset damping_kwdset(sizeof(dampingstr)/sizeof(*dampingstr), dampingstr);



// aliases used for properties in preprocessor
enum proptype {ndofs=1, boundarycond=2, loadnodes=3, crosssec=4, localsystem=5,
               eltype=6, matel=7, loadelems=8, dloadnodes=9, dloadelems=10,
               initcond=11, comcodnum=12, loadedge=13, sscomp=14, nodetemp=15, 
               timefun=16,surfaceload=17, elsurfaceload=18, nodtimefunc=19, 
               eltimefunc=20};



// aliases for element types
enum elemtype {bar2d=1,beam2d=2,bar3d=3,beam3d=4,beamg3d=5,barq2d=6,barq3d=7,subsoilbeam=8,beam2dsp=9,
               spring_1 = 10, spring_2 = 11, spring_3 = 12,
               spring_4 = 13, spring_5 = 14, spring_6 = 15,
               planeelementlt=20,planeelementqt=21,planeelementrotlt=22,
               planeelementlq=23,planeelementqq=24,planeelementrotlq=25,
               planeelementsubqt=30,planequadcontact=35,
               cctel=41,dktel=42,dstel=43,q4plateel=45,subsoilplatetr=50,subsoilplateq=51,
               axisymmlt=60,axisymmlq=63,axisymmqq=64,
               shelltrelem=80,shellqelem=81,
               lineartet=100,linearhex=102,quadrhex=103,lineartetrot=104,linearhexrot=105,
	       linearwed=106,quadrwed=107,
               particleelem=200};
const enumstr elemtypestr[] = {{"bar2d",1}, {"beam2d",2}, {"bar3d",3}, {"beam3d",4}, {"beamg3d",5}, 
                               {"barq2d",6},{"barq3d",7}, {"subsoilbeam",8}, {"beam2dsp",9},
                               {"spring_1",10}, {"spring_2",11}, {"spring_3",12},
                               {"spring_4",13}, {"spring_5",14}, {"spring_6",15},
                               {"planeelementlt",20}, {"planeelementqt",21}, {"planeelementrotlt",22},
                               {"planeelementlq",23}, {"planeelementqq",24}, {"planeelementrotlq",25},
                               {"planeelementsubqt",30}, {"planequadcontact",35},
                               {"cctel",41}, {"dktel",42}, {"dstel",43}, {"q4plateel",45},
                               {"subsoilplatetr",50}, {"subsoilplateq",51},
			       {"axisymmlt",60}, {"axisymmlq",63}, {"axisymmqq",64},
			       {"shelltrelem",80}, {"shellqelem",81},
			       {"lineartet",100},  {"linearhex",102}, {"quadrhex",103},
			       {"lineartetrot",104}, {"linearhexrot",105},
			       {"linearwed",106}, {"quadrwed",107},
                               {"particleelem",200}};
const kwdset elemtype_kwdset(sizeof(elemtypestr)/sizeof(*elemtypestr), elemtypestr);



// aliases for strain/stress state
enum strastrestate {bar=1,plbeam=2,spacebeam=5,
                    planestress=10,planestrain=11,platek=15,plates=16,
                    axisymm=20,shell=25,
                    spacestress=30};
const enumstr strastrestatestr[] = {{"bar",1}, {"plbeam",2}, {"spacebeam",5},
                                    {"planestress",10}, {"planestrain",11}, {"platek",15}, {"plates",16},
                                    {"axisymm",20}, {"shell",25}, {"spacestress",30}};
const kwdset strastrestate_kwdset(sizeof(strastrestatestr)/sizeof(*strastrestatestr), strastrestatestr);



// aliases for  material model classes
enum materialmodel {local=0,nonlocal=1};
const enumstr materialmodelstr[] = {{"local",0}, {"nonlocal",1}};
const kwdset materialmodel_kwdset(sizeof(materialmodelstr)/sizeof(*materialmodelstr), materialmodelstr);



// aliases for material models
enum mattype {elisomat=1,elgmat3d=2,
              simplas1d=10,jflow=11,jflow2=12,mohrcoulparab=22,mohrcoul=24,
              boermaterial=25,druckerprager=26,modcamclaymat=30,
              shefpl=40,chenplast=42,
              microplaneM4=50,microsimp=51,microfibro=52,
              simvisplas=70,simvisc=71,lemaitr=75,
              scaldamage=100,scaldamagecc=101,glasgowdamage=102,scaldamage_time=103, anisodamage=104, anisodamagerot=105, ortodamage=106,
              contmat=150,
              aci=200,cebfip=202,
              nonlocplastmat=310,nonlocdamgmat=320,nonlocalmicroM4=340,
              graphm=400,geoelast=401,graphmtime=402,
              creepbaz=500,creepb3=502,creepdpl=503,creeprs=504,creepeffym=505,winklerpasternak=550,consolidation=600,
              glasgowmechmat=700,
              rcmatmodnorm=800,
              therisodilat=900, therisodilattime=910,
              damage_plasticity=1000, viscoplasticity=1010, elasttimemat=1020, 
              creep_damage=1030, time_switchmat=1040,
              lenjonespot=2000};
const enumstr mattypestr[] = {{"elisomat",1}, {"elgmat3d",2}, {"simplas1d",10}, 
                              {"jflow",11}, {"jflow2",12}, {"mohrcoulparab",22}, 
                              {"mohrcoul",24}, {"boermaterial",25}, {"druckerprager",26}, 
                              {"modcamclaymat",30}, {"shefpl",40}, {"chenplast",42}, 
                              {"microplaneM4",50}, {"microsimp",51}, {"microfibro",52}, 
                              {"simvisplas",70}, {"simvisc",71}, {"lemaitr",75}, 
                              {"scaldamage",100}, {"scaldamagecc",101}, {"glasgowdamage",102}, 
                              {"scaldamage_time",103}, {"anisodamage",104}, {"anisodamagerot",105}, {"ortodamage", 106},
                              {"contmat",150}, 
                              {"aci",200}, {"cebfip",202}, {"nonlocplastmat",310}, 
                              {"nonlocdamgmat",320}, {"nonlocalmicroM4",340}, {"graphm",400}, 
                              {"geoelast",401}, {"graphmtime",402}, {"creepbaz",500}, 
                              {"creepb3",502}, {"creepdpl",503}, {"creeprs",504}, 
                              {"winklerpasternak",550}, {"consolidation",600}, {"glasgowmechmat",700}, 
                              {"rcmatmodnorm",800}, {"therisodilat",900}, {"therisodilattime",910}, 
                              {"damage_plasticity",1000}, {"viscoplasticity",1010}, {"elasttimemat",1020},  
                              {"creep_damage",1030}, {"time_switchmat",1040}, {"lenjonespot",2000}};
const kwdset mattype_kwdset(sizeof(mattypestr)/sizeof(*mattypestr), mattypestr);



// aliases for cross-section 
enum crsectype {nocrosssection=0, csbar2d=1,csbeam2d=2,csbeam3d=4,csplanestr=10,cs3dprob=20,csnodal=40};
const enumstr crsectypestr[] = {{"nocrosssection",0}, {"csbar2d",1}, {"csbeam2d",2}, {"csbeam3d",4}, 
                                {"csplanestr",10}, {"cs3dprob",20}, {"csnodal",40}};
const kwdset crsectype_kwdset(sizeof(crsectypestr)/sizeof(*crsectypestr), crsectypestr);



//aliase for load of element
enum elloadtype {volume=1, edge=2,surface=3};
const enumstr elloadtypestr[] = {{"volume",1}, {"edge",2}, {"surface",3}};
const kwdset elloadtype_kwdset(sizeof(elloadtypestr)/sizeof(*elloadtypestr), elloadtypestr);



// aliases for point position on element
enum elemposition {nowhere=0,intpts=1,enodes=2,cenodes=3,userdefined=10};
const enumstr elempositionstr[] = {{"nowhere",0}, {"intpts",1}, {"enodes",2}, {"cenodes",3}, {"userdefined",10}};
const kwdset elemposition_kwdset(sizeof(elempositionstr)/sizeof(*elempositionstr), elempositionstr);



// aliases for quantity types
enum strastre {strain=0,stress=1,other=2,pstrain=3, pstress=4};
const enumstr strastrestr[] = {{"strain",0}, {"stress",1}, {"other",2}, {"pstrain",3}, {"pstress",4}};
const kwdset strastre_kwdset(sizeof(strastrestr)/sizeof(*strastrestr), strastrestr);



//aliases for intial conditions
enum inictype {none = 0, inidisp = 1, inistrain = 2, inistress = 4, iniother = 8, inicond = 16};
const enumstr inictypestr[] = {{"none",0}, {"inidisp",1}, {"inistrain",2}, 
                                {"inistress",4}, {"iniother",8}, {"inicond",16}};
const kwdset inictype_kwdset(sizeof(inictypestr)/sizeof(*inictypestr), inictypestr);



// aliases for types of equivalent strain computation
enum paramf_type {norstrain = 1,    norenergy = 2,
                  norposstrain = 3, norposenergy = 4,
                  norrankine = 5,   norrankinesmooth = 6,
                  normazar = 7, vonmises= 8};
const enumstr paramf_typestr[] = {{"norstrain",1}, {"norenergy",2}, {"norposstrain",3}, {"norposenergy",4},
                                  {"norrankine",5}, {"norrankinesmooth",6}, {"normazar",7}, {"vonmises",8}};
const kwdset paramf_type_kwdset(sizeof(paramf_typestr)/sizeof(*paramf_typestr), paramf_typestr);



// aliases for switching on/off of correction of dissipated energy
enum corr_disip_en {corr_off = 0, corr_on = 1}; 
const enumstr corr_disip_enstr[] = {{"corr_off",0}, {"corr_on",1}}; 
const kwdset corr_disip_en_kwdset(sizeof(corr_disip_enstr)/sizeof(*corr_disip_enstr), corr_disip_enstr);



// aliases for evolution function of damage parmetrs in anisotropic damage model
enum dam_evolfunc {brittle = 1,  quasi_brittle= 2}; 
const enumstr dam_evolfuncstr[] = {{"brittle",1}, {"quasi_brittle",2}}; 
const kwdset dam_evolfunc_kwdset(sizeof(dam_evolfuncstr)/sizeof(*dam_evolfuncstr), dam_evolfuncstr);



// aliases for type of averaging in nonlocal material models
enum wavrg {avggamma = 1, avgepsp = 2};                  
const enumstr wavrgstr[] = {{"avggamma",1}, {"avgepsp",2}};
const kwdset wavrg_kwdset(sizeof(wavrgstr)/sizeof(*wavrgstr), wavrgstr);



// aliases for types of averaging functions in nonlocal material models
enum avrgf {parab = 1, cubic = 2};
const enumstr avrgfstr[] = {{"parab",1}, {"cubic",2}};
const kwdset avrgf_kwdset(sizeof(avrgfstr)/sizeof(*avrgfstr), avrgfstr);



// aliases for types of functions for modulus evolution in elasttime model
enum ym_evolfunc {b3law = 1,doublepwrlaw = 2,userfunc = 3};
const enumstr ym_evolfuncstr[] = {{"b3law",1}, {"doublepwrlaw",2}, {"userfunc",3}};
const kwdset ym_evolfunc_kwdset(sizeof(ym_evolfuncstr)/sizeof(*ym_evolfuncstr), ym_evolfuncstr);



// aliases for format of graphics output
enum graphfmt {grfmt_no=0, grfmt_open_dx=1, grfmt_femcad=2, grfmt_gid=3, grfmt_gid_sep=4};
const enumstr graphfmtstr[] = {{"grfmt_no",0}, {"grfmt_open_dx",1}, {"grfmt_femcad",2}, 
                                {"grfmt_gid",3}, {"grfmt_gid_sep",4}};
const kwdset graphfmt_kwdset(sizeof(graphfmtstr)/sizeof(*graphfmtstr), graphfmtstr);



// aliases for type of printed unknown
enum prunk {pr_displ=1, pr_strains=2, pr_stresses=3, pr_forces=4, pr_react=5, 
            pr_stepid=6, pr_appload=7, pr_other=8};
const enumstr prunkstr[] = {{"pr_displ",1}, {"pr_strains",2}, {"pr_stresses",3}, {"pr_forces",4}, 
                             {"pr_react",5}, {"pr_stepid",6}, {"pr_appload",7}, {"pr_other",8}};
const kwdset prunk_kwdset(sizeof(prunkstr)/sizeof(*prunkstr), prunkstr);



//aliases for time printing
enum timetypeprin {seconds=1,minutes=2,hours=3,days=4};



// aliases for nonmechanical quantity 
enum nonmechquant {temperature=1,moist=2,initial_temperature=3};
const enumstr nonmechquantstr[] = {{"temperature",1}, {"moist",2},{"initial_temperature",3}};
const kwdset nonmechquant_kwdset(sizeof(nonmechquantstr)/sizeof(*nonmechquantstr), nonmechquantstr);



// aliases for types of timedependent load
enum dynload {timeindload = 1, seismicload=10, responsespectrum=11, timedepload=20};
const enumstr dynloadstr[] = {{"timeinddload",1}, {"seismicload",10}, {"responsespectrum",11}, {"timedepload",20}};
const kwdset dynload_kwdset(sizeof(dynloadstr)/sizeof(*dynloadstr), dynloadstr);



// aliases for directions of dynamic load
enum dirdynload {xdir=1,ydir=2,zdir=3};
const enumstr dirdynloadstr[] = {{"xdir",1}, {"ydir",2}, {"zdir",3}};
const kwdset dirdynload_kwdset(sizeof(dirdynloadstr)/sizeof(*dirdynloadstr), dirdynloadstr);



// aliases types of eigenstrains
enum eigstraintype {eigstrain=1,tempstrain=2};
const enumstr eigstraintypestr[] = {{"eigstrain",1}, {"tempstrain",2}};
const kwdset eigstraintype_kwdset(sizeof(eigstraintypestr)/sizeof(*eigstraintypestr), eigstraintypestr);



// aliases for types of graph approximation in graphmat material models
enum graphtype { glinear = 0, gtable = 1, gfunc = 2, gfunc_ser = 3};
const enumstr graphtypestr[] = {{"glinear",0}, {"gtable",1}, {"gfunc",2}, {"gfunc_ser",3}};
const kwdset graphtype_kwdset(sizeof(graphtypestr)/sizeof(*graphtypestr), graphtypestr);



// aliases for different types of hardening function in plasticity material models
enum hardensoften {nohs=0,linearhs=1,sqrths=2,sinhs=3,sqrtsinhs=4,
                   anlinearhs=101,ansqrths=102,ansinhs=103,ansqrtsinhs=104,
                   plstrainnorm=201,limplstrainnorm=202};
const enumstr hardensoftenstr[] = {{"nohs",0}, {"linearhs",1}, {"sqrths",2}, {"sinhs",3}, {"sqrtsinhs",4},
                                     {"anlinearhs",101}, {"ansqrths",102}, {"ansinhs",103}, {"ansqrtsinhs",104},
                                     {"plstrainnorm",201}, {"limplstrainnorm",202}};
const kwdset hardensoften_kwdset(sizeof(hardensoftenstr)/sizeof(*hardensoftenstr), hardensoftenstr);



// aliases for input variable to hardening/softening
enum hsinputvar {plstrnorm=1,consistpar=2,limplstrnorm=11,limconsistpar=12,limanplstrnorm=21,limanconsistpar=22};
const enumstr hsinputvarstr[] = {{"plstrnorm",1}, {"consistpar",2}, {"limplstrnorm",11}, {"limconsistpar",12},
                                  {"limanplstrnorm",21}, {"limanconsistpar",22}};
const kwdset hsinputvar_kwdset(sizeof(hsinputvarstr)/sizeof(*hsinputvarstr), hsinputvarstr);



// aliases for ?
enum strestate {notdef=0,tension=1,compression=2};
const enumstr strestatestr[] = {{"notdef",0}, {"tension",1}, {"compression",2}};
const kwdset strestate_kwdset(sizeof(strestatestr)/sizeof(*strestatestr), strestatestr);



// aliases for ?
enum integratedquant {locstress=1,nonlocstress=2,stressincr=3,eigstress=10};
const enumstr integratedquantstr[] = {{"locstress",1}, {"nonlocstress",2}, {"stressincr",3}, {"eigstress",10}};
const kwdset integratedquant_kwdset(sizeof(integratedquantstr)/sizeof(*integratedquantstr), integratedquantstr);


// aliases for keyword processing switch
enum pkwd_sw {nokwd=0, kwd_pd=1, kwd_outd=2, kwd_pd_outd=3, kwd_full=4};


#endif


