#include "shelltr.h"
#include "global.h"
#include "globmat.h"
#include "genfile.h"
#include "node.h"
#include "element.h"
#include "plelemrotlt.h"
#include "dkt.h"
#include "intpoints.h"
#include <math.h>

shelltr::shelltr (void)
{
  long i,j;
  
  if (Perlt==NULL)   Perlt = new planeelemrotlt;
  if (Dkt==NULL)     Dkt = new dktelem;

  //  number nodes on element
  nne=3;
  //  number of DOFs on element
  ndofe=18;
  //  number of strain/stress components
  tncomp=6;
  //  number of functions approximated
  napfun=6;
  //  number of edges on element
  ned=3;
  //  number of nodes on one edge
  nned=2;
  //  strain/stress state
  ssst=shell;
  
  //  number of blocks (parts of geometric matrix)
  //  nb = Perlt->nb + Cct->nb;
  nb = Perlt->nb + Dkt->nb;
  
  ncomps = new long [Perlt->nb];
  ncomps[0]=2;
  ncomps[1]=1;
 
  ncompd = new long [Dkt->nb];
  ncompd[0]=3;

  nip = new long* [nb];
  intordsm = new long* [nb];
  for (i=0;i<nb;i++){
    nip[i]=new long [nb];
    intordsm[i] = new long [nb];
  }
  tncomps=Perlt->tncomp;
  tncompd=Dkt->tncomp;
  
  long a00,a01,a10,a11,b00;
  //long b01,b10,b11;

  a00=Perlt->nip[0][0];  a01=Perlt->nip[0][1];
  a10=Perlt->nip[1][0];  a11=Perlt->nip[1][1];
  b00=Dkt->nip[0][0];

  nip[0][0]=a00;  nip[0][1]=a01;  nip[0][2]=0;    
  nip[1][0]=a10;  nip[1][1]=a11;  nip[1][2]=0; 
  nip[2][0]=0;    nip[2][1]=0;    nip[2][2]=b00;
  
  a00=Perlt->intordsm[0][0];  a01=Perlt->intordsm[0][1];
  a10=Perlt->intordsm[1][0];  a11=Perlt->intordsm[1][1];
  b00=Dkt->intordsm[0][0];
  
  intordsm[0][0]=a00;  intordsm[0][1]=a01; intordsm[0][2]=0;
  intordsm[1][0]=a10;  intordsm[1][1]=a11; intordsm[1][2]=0;
  intordsm[2][0]=0;    intordsm[2][1]=0;   intordsm[2][2]=b00;

  dofe = new long [2];
  dofe[0]=Perlt->ndofe;
  dofe[1]=Dkt->ndofe;
  
  tnip=0;
  for (i=0;i<nb;i++){
    for (j=0;j<nb;j++){
      tnip+=nip[i][j];
    }
  }
  
}

shelltr::~shelltr (void)
{
  long i;
  
  for (i=0;i<nb;i++){
    delete [] nip[i];
  }
  delete [] nip;
  delete [] ncomps;
  delete [] ncompd;
}

void shelltr::eleminit (long eid)
{

  long ii,jj;
  Mt->elements[eid].nb=nb;
  Mt->elements[eid].intordsm = new long* [nb];
  Mt->elements[eid].nip = new long* [nb];

  for (ii=0;ii<nb;ii++){
    Mt->elements[eid].intordsm[ii] = new long [nb];
    Mt->elements[eid].nip[ii] = new long [nb];
    for (jj=0;jj<nb;jj++){
      Mt->elements[eid].intordsm[ii][jj]=intordsm[ii][jj];
      Mt->elements[eid].nip[ii][jj]=nip[ii][jj];
    }
  }

}

/**
   function assembles transformation %matrix T between coordinate system
   defined on element and global coordinate system
      
   T x_local = x_global
   x_local = T^T x_global
   
   direction %vector denotes %vector defining local coordinate axis
   
   @param x,y - vectors of nodal coordinates in element coordinate system
   @param tran - tranformation %matrix T
   @param gx,gy,gz - vectors of nodal coordinates in global coordinate systems
   
   JK, 1.10.2008
*/
void shelltr::tran_mat(vector &x, vector &y, matrix &tran, vector &gx, vector &gy, vector &gz)
{
  double c,p,l;
  
  //  first direction vector is located in the first column
  tran[0][0]=gx[1]-gx[0];
  tran[1][0]=gy[1]-gy[0];
  tran[2][0]=gz[1]-gz[0];

  //  length of the first direction vector
  l=sqrt(tran[0][0]*tran[0][0]+tran[1][0]*tran[1][0]+tran[2][0]*tran[2][0]);
  
  //  normalization of the first direction vector
  tran[0][0]/=l;
  tran[1][0]/=l;
  tran[2][0]/=l;


  //  second direction vector is located in the second column
  tran[0][1]=gx[2]-gx[0];
  tran[1][1]=gy[2]-gy[0];
  tran[2][1]=gz[2]-gz[0];
  
  //  dot product of the first and second direction vector
  p=tran[0][0]*tran[0][1]+tran[1][0]*tran[1][1]+tran[2][0]*tran[2][1];
  //  coefficient in Gramm-Schmidt orthonormalization process
  c=0.0-p;
  
  //  orthogonalization of the second vector with respect to the first vector
  tran[0][1]=tran[0][1]+tran[0][0]*c;
  tran[1][1]=tran[1][1]+tran[1][0]*c;
  tran[2][1]=tran[2][1]+tran[2][0]*c;
  
  //  length of the second direction vector
  l=sqrt(tran[0][1]*tran[0][1]+tran[1][1]*tran[1][1]+tran[2][1]*tran[2][1]);

  //  normalization of the second direction vector
  tran[0][1]/=l;
  tran[1][1]/=l;
  tran[2][1]/=l;

  
  //  third direction vector is obtained from vector product of the first and second vector
  tran[0][2]=tran[1][0]*tran[2][1]-tran[2][0]*tran[1][1];
  tran[1][2]=tran[2][0]*tran[0][1]-tran[0][0]*tran[2][1];
  tran[2][2]=tran[0][0]*tran[1][1]-tran[1][0]*tran[0][1];
  
  //  length of the third vector (it should be one, it is calculated in order to improve numerical stability)
  l=sqrt(tran[0][2]*tran[0][2]+tran[1][2]*tran[1][2]+tran[2][2]*tran[2][2]);
  
  //  normalization of the third vector
  tran[0][2]/=l;
  tran[1][2]/=l;
  tran[2][2]/=l;
  
  // *****************************************************************
  //  transformation of nodal coordinates to element coordinate system
  // *****************************************************************
  
  //  transformation of coordinates of the first node
  x[0]=tran[0][0]*gx[0]+tran[1][0]*gy[0]+tran[2][0]*gz[0];
  y[0]=tran[0][1]*gx[0]+tran[1][1]*gy[0]+tran[2][1]*gz[0];
  c   =tran[0][2]*gx[0]+tran[1][2]*gy[0]+tran[2][2]*gz[0];
  
  //  transformation of coordinates of the second node
  x[1]=tran[0][0]*gx[1]+tran[1][0]*gy[1]+tran[2][0]*gz[1];
  y[1]=tran[0][1]*gx[1]+tran[1][1]*gy[1]+tran[2][1]*gz[1];
  c   =tran[0][2]*gx[1]+tran[1][2]*gy[1]+tran[2][2]*gz[1];
  
  //  transformation of coordinates of the third node
  x[2]=tran[0][0]*gx[2]+tran[1][0]*gy[2]+tran[2][0]*gz[2];
  y[2]=tran[0][1]*gx[2]+tran[1][1]*gy[2]+tran[2][1]*gz[2];
  c   =tran[0][2]*gx[2]+tran[1][2]*gy[2]+tran[2][2]*gz[2];
}

/**
   function assembles transformation %matrix between coordinate system
   defined on element and global coordinate system
   transformation deals with coordinates
   
   transformation %matrix from local to global systems x,y,z
   In vectors   s  are global coordinates of Nodes 1 , 2 , 3
   Vector local s. s.( edge 1-2 = x' )  in glob. s. s.
   
   v(g)  = tran * v(l)
               t                t
   v(l)  = tran  * v(g)  =  v(g)  * tran
   
                               t
   sm(g) = tran  * sm(l) * tran
               t
   sm(l) = tran  * sm(g) * tran
   
*/
void shelltr::tran_mat_old (vector &x, vector &y, matrix &tran, vector &gx, vector &gy, vector &gz)
{
  long i,i1;
  double dl;
  matrix a(3,3);
 
  for (i=0; i<3; i++) {
    i1=i+1; if(i1>2)i1=i1-3;
//    a[i][0]=s2[i]-s1[i];
    a[0][i]=gx[i1]-gx[i];
//    a[i][1]=s3[i]-s2[i];
    a[1][i]=gy[i1]-gy[i];
//    a[i][2]=s1[i]-s3[i];
    a[2][i]=gz[i1]-gz[i];
  }
  
  dl=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);

  tran[0][0]=a[0][0]/dl;
  tran[1][0]=a[1][0]/dl;
  tran[2][0]=a[2][0]/dl;
  
  tran[0][2]=a[1][0]*a[2][1]-a[2][0]*a[1][1];
  tran[1][2]=a[2][0]*a[0][1]-a[0][0]*a[2][1];
  tran[2][2]=a[0][0]*a[1][1]-a[1][0]*a[0][1];

  dl=sqrt(tran[0][2]*tran[0][2]+tran[1][2]*tran[1][2]+tran[2][2]*tran[2][2]);

  tran[0][2]=tran[0][2]/dl;
  tran[1][2]=tran[1][2]/dl;
  tran[2][2]=tran[2][2]/dl;
  
  tran[0][1]=tran[1][2]*tran[2][0]-tran[2][2]*tran[1][0];
  tran[1][1]=tran[2][2]*tran[0][0]-tran[0][2]*tran[2][0];
  tran[2][1]=tran[0][2]*tran[1][0]-tran[1][2]*tran[0][0];

  dl=sqrt(tran[0][1]*tran[0][1]+tran[1][1]*tran[1][1]+tran[2][1]*tran[2][1]);

  tran[0][1]=tran[0][1]/dl;
  tran[1][1]=tran[1][1]/dl;
  tran[2][1]=tran[2][1]/dl;
/*  
// local coordinate x=s12
    sl1[0]=0.0;    sl1[1]=0.0;    sl1[2]=0.0;
    sl2[0]=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);
    sl2[1]=0.0;    sl2[2]=0.0;
    sl3[0]=(s3(0)-s1(0))*tran(0,0)+(s3(1)-s1(1))*tran(1,0)+(s3(2)-s1(2))*tran(2,0);
    sl3[1]=(s3(0)-s1(0))*tran(0,1)+(s3(1)-s1(1))*tran(1,1)+(s3(2)-s1(2))*tran(2,1);
    sl3[2]=0.0;
*/
// local coordinate x=s12
    x[0]=0.0;
    y[0]=0.0;
    x[1]=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);
    y[1]=0.0;
    x[2]=(gx(2)-gx(0))*tran(0,0)+(gy(2)-gy(0))*tran(1,0)+(gz(2)-gz(0))*tran(2,0);
    y[2]=(gx(2)-gx(0))*tran(0,1)+(gy(2)-gy(0))*tran(1,1)+(gz(2)-gz(0))*tran(2,1);
}





/**
   function assembles transformation %matrix between global coordinate system
   and local nodal coordinate sytem
   transformation deals with nodal forces

   transformation %matrix x_g = T x_l

   9.5.2002
*/
void shelltr::transf_matrix (ivector &nodes,matrix &tmat)
{
  long i,i6,n,m;

  fillm (0.0,tmat);

  n=nodes.n;
  m=tmat.m;
  for (i=0;i<m;i++){
    tmat[i][i]=1.0;
  }
  
  for (i=0;i<n;i++){
    if (Mt->nodes[nodes[i]].transf>0){
	  i6=i*6;
      tmat[i6][i6]   = Mt->nodes[nodes[i]].e1[0];  tmat[i6][i6+1]   = Mt->nodes[nodes[i]].e2[0]; tmat[i6][i6+2]     = Mt->nodes[nodes[i]].e3[0];
      tmat[i6+1][i6] = Mt->nodes[nodes[i]].e1[1];  tmat[i6+1][i6+1] = Mt->nodes[nodes[i]].e2[1]; tmat[i6+1][i6+2]   = Mt->nodes[nodes[i]].e3[1];
      tmat[i6+2][i6] = Mt->nodes[nodes[i]].e1[2];  tmat[i6+2][i6+1] = Mt->nodes[nodes[i]].e2[2]; tmat[i6+2][i6+2]   = Mt->nodes[nodes[i]].e3[2];
	  i6=i*6+3;
      tmat[i6][i6]   = Mt->nodes[nodes[i]].e1[0];  tmat[i6][i6+1]   = Mt->nodes[nodes[i]].e2[0]; tmat[i6][i6+2]     = Mt->nodes[nodes[i]].e3[0];
      tmat[i6+1][i6] = Mt->nodes[nodes[i]].e1[1];  tmat[i6+1][i6+1] = Mt->nodes[nodes[i]].e2[1]; tmat[i6+1][i6+2]   = Mt->nodes[nodes[i]].e3[1];
      tmat[i6+2][i6] = Mt->nodes[nodes[i]].e1[2];  tmat[i6+2][i6+1] = Mt->nodes[nodes[i]].e2[2]; tmat[i6+2][i6+2]   = Mt->nodes[nodes[i]].e3[2];
    }
  }
}


/**
   function assembles stiffness %matrix of triangular shell element
   
   @param eid - element id
   @param sm - stiffness %matrix
   
   4.5.2002
*/
void shelltr::res_stiffness_matrix (long eid,matrix &sm)
{
  long *rcn,*ccn,transf;
  vector gx(nne),gy(nne),gz(nne),x(nne),y(nne);
  ivector nodes(nne);
  matrix lsm,tran(3,3);
  
  Mt->give_elemnodes (eid,nodes);
  Mt->give_node_coord3d (gx,gy,gz,eid);
  //  funkce transformujici x,y,z do roviny xy 
  tran_mat(x,y, tran, gx,  gy,  gz);
  
  fillm (0.0,sm);
  
  rcn = new long [dofe[0]];
  ccn = new long [dofe[0]];
  allocm (dofe[0],dofe[0],lsm);
  
  //  contribution from triangular plane element with rotational degrees of freedom
  Perlt->stiffness_matrix (eid,0,0,lsm,x,y);
  
  rcn[0]=1;  rcn[1]=2;  rcn[2]=6;  rcn[3]=7;  rcn[4]=8;  rcn[5]=12;  rcn[6]=13;  rcn[7]=14;  rcn[8]=18;
  ccn[0]=1;  ccn[1]=2;  ccn[2]=6;  ccn[3]=7;  ccn[4]=8;  ccn[5]=12;  ccn[6]=13;  ccn[7]=14;  ccn[8]=18;
  
  mat_localize (sm,lsm,rcn,ccn);

  delete [] ccn;  delete [] rcn;  destrm (lsm);
  
  rcn = new long [dofe[1]];
  ccn = new long [dofe[1]];
  allocm (dofe[1],dofe[1],lsm);
  
  //  contribution from constant curve triangular plate element
  //Cct->stiffness_matrix (eid,2,2,lsm,x,y);
  Dkt->stiffness_matrix (eid,2,2,lsm,x,y);
  
  rcn[0]=3;  rcn[1]=4;  rcn[2]=5;  rcn[3]=9;  rcn[4]=10;  rcn[5]=11;  rcn[6]=15;  rcn[7]=16;  rcn[8]=17;
  ccn[0]=3;  ccn[1]=4;  ccn[2]=5;  ccn[3]=9;  ccn[4]=10;  ccn[5]=11;  ccn[6]=15;  ccn[7]=16;  ccn[8]=17;

  mat_localize (sm,lsm,rcn,ccn);

  //  transformation of stiffness matrix to global coordinate system
  lgmatrixtransfblock(sm, tran);
  //  transformation of stiffness matrix to nodal coordinate system
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glmatrixtransf (sm,tmat);
  }
  /*
  fprintf (Out,"\n\n prvek cislo %ld",eid);
  for (long i=0;i<ndofe;i++){
    fprintf (Out,"\n %10.7e %10.7e %10.7e %10.7e %10.7e %10.7e",sm[i][0],sm[i][1],sm[i][2],sm[i][3],sm[i][4],sm[i][5]);
    fprintf (Out,"\n %10.7e %10.7e %10.7e %10.7e %10.7e %10.7e",sm[i][6],sm[i][7],sm[i][8],sm[i][9],sm[i][10],sm[i][11]);
    fprintf (Out,"\n %10.7e %10.7e %10.7e %10.7e %10.7e %10.7e",sm[i][12],sm[i][13],sm[i][14],sm[i][15],sm[i][16],sm[i][17]);
  }
 */ 
  delete [] ccn;  delete [] rcn;  destrm (lsm);
  
}



/**
   function computes strains at integration points of element
   
   @param lcid - load case id
   @param eid - element id
   
   JK, 23.9.2008
*/
void shelltr::res_ip_strains (long lcid,long eid)
{
  long *ccn;
  vector aux,x(nne),y(nne),gx(nne),gy(nne),gz(nne),r(ndofe),v;
  ivector cn(ndofe),nodes(nne);
  matrix tmat,tran(3,3);
  
  Mt->give_elemnodes (eid,nodes);
  Mt->give_node_coord3d (gx,gy,gz,eid);
  //  funkce transformujici x,y,z do roviny xy 
  tran_mat(x,y, tran, gx,  gy,  gz);
  Mt->give_code_numbers (eid,cn.a);
  eldispl (lcid,eid,r.a,cn.a,ndofe);
  
  //  transformation of displacement vector from nodal coordinate system to the global coordinate system
  long transf = Mt->locsystems (nodes);
  if (transf>0){
    allocv (ndofe,aux);
    allocm (ndofe,ndofe,tmat);
    transf_matrix (nodes,tmat);
    //locglobtransf (aux,r,tmat);
    lgvectortransf (aux,r,tmat);
    copyv (aux,r);
    destrv (aux);
    destrm (tmat);
  }
  
  //  transformation from global to element coordinate system
  glvectortransfblock (r,tran);
  
  ccn = new long [dofe[0]];
  allocv (dofe[0],v);
  ccn[0]=1;  ccn[1]=2;  ccn[2]=6;  ccn[3]=7;  ccn[4]=8;  ccn[5]=12;  ccn[6]=13;  ccn[7]=14;  ccn[8]=18;
  globloc (r.a,v.a,ccn,dofe[0]);

  Perlt->ip_strains (lcid,eid,0,0,x,y,v);
  
  /*
  vector eps(6);
  long ri=0;
  long ci=0;
  long ipp=Mt->elements[eid].ipp[ri][ci];
  for (long i=0;i<6;i++){
    eps[0]=Mm->ip[ipp].strain[0];
    eps[1]=Mm->ip[ipp].strain[1];
    eps[5]=Mm->ip[ipp].strain[2];
    loc_glob_tens_trans (eps,tran);
    Mm->ip[ipp].strain[0]=eps[0];
    Mm->ip[ipp].strain[1]=eps[1];
    Mm->ip[ipp].strain[2]=eps[5];
    ipp++;
  }
  */

  delete [] ccn;
  destrv (v);

  allocv (dofe[1],v);
  ccn = new long [dofe[1]];
  ccn[0]=3;  ccn[1]=4;  ccn[2]=5;  ccn[3]=9;  ccn[4]=10;  ccn[5]=11;  ccn[6]=15;  ccn[7]=16;  ccn[8]=17;
  globloc (r.a,v.a,ccn,dofe[1]);

  Dkt->ip_strains (lcid,eid,2,2,x,y,v);

  destrv (v);
  delete [] ccn;

}


/**
   function computes stresses at integration points
   
   @param lcid - load case id
   @param eid - element id
   
   JK, 26.9.2008
*/
void shelltr::res_ip_stresses (long lcid,long eid)
{
  Perlt->compute_nlstress (lcid,eid,0,0);
  Dkt->compute_nlstress (lcid,eid,2,2);
}

/**
   function computes stresses
   
   @param lcid - load case id
   @param eid - element id
   
   JK, 26.9.2008
*/
void shelltr::compute_nlstress (long lcid,long eid)
{
  Perlt->compute_nlstress (lcid,eid,0,0);
  Dkt->compute_nlstress (lcid,eid,2,2);
}


/**
   function computes correct increments of stresses at integration points on element

   @param lcid - number of load case
   @param eid - element id
   
   JK, 23.9.2008
*/
void shelltr::compute_nlstressincr (long lcid,long eid)
{
  Perlt->compute_nlstressincr (lcid,eid,0,0);
  Dkt->compute_nlstressincr (lcid,eid,2,2);
}


/**
   function computes resulting internal forces
   
   @param lcid - load case id
   @param eid - element id
   @param ifor - %vector of internal forces
   
   JK, 26.9.2008
*/
void shelltr::res_internal_forces (long lcid,long eid,vector &ifor)
{
  long *ccn,transf;
  ivector nodes (nne);
  vector v(ndofe),x(nne),y(nne),z(nne),gx(nne),gy(nne),gz(nne);
  matrix tran(3,3);
  
  Mt->give_node_coord3d (gx,gy,gz,eid);
  //  funkce transformujici x,y,z do roviny xy 
  tran_mat(x,y, tran, gx,  gy,  gz);
  
  ccn = new long [dofe[0]];
  allocv (dofe[0],v);
  
  Perlt->internal_forces (lcid,eid,0,0,v,x,y);
  
  ccn[0]=1;  ccn[1]=2;  ccn[2]=6;  ccn[3]=7;  ccn[4]=8;  ccn[5]=12;  ccn[6]=13;  ccn[7]=14;  ccn[8]=18;
  locglob (ifor.a,v.a,ccn,dofe[0]);
  
  delete [] ccn;  destrv (v);
  
  ccn = new long [dofe[1]];
  allocv (dofe[1],v);
  
  Dkt->internal_forces (lcid,eid,2,2,v,x,y);
  
  ccn[0]=3;  ccn[1]=4;  ccn[2]=5;  ccn[3]=9;  ccn[4]=10;  ccn[5]=11;  ccn[6]=15;  ccn[7]=16;  ccn[8]=17;
  locglob (ifor.a,v.a,ccn,dofe[1]);
  
  
  //  transformation of nodal forces
  //  (in the case of nodal coordinate systems)
  Mt->give_elemnodes (eid,nodes);
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glvectortransf (ifor,v,tmat);
    copyv (v,ifor);
  }
}





void shelltr::nod_strains_ip (long lcid,long eid)
{
  
  //  contribution from triangular plane element with rotational degrees of freedom
  //Perlt->nod_strains_ip (lcid,eid,0,0);
  //  contribution from triangular plate element
  //Dkt->nod_strains_ip (lcid,eid,2,2);

  long i,j,ipp;
  ivector ipnums(nne),ipnump(nne),nod(nne);
  vector gx(nne),gy(nne),gz(nne),x(nne),y(nne);
  vector epss(tncomps),epsp(tncomps),eps(tncomp);
  matrix tran(3,3);

  //  node coordinates in global coordinate system
  Mt->give_node_coord3d (gx,gy,gz,eid);
  //  transformation from global to element coordinate system
  tran_mat(x,y, tran, gx,  gy,  gz);
  
  //  numbers of integration points closest to nodes
  //  (function is from the file GEFEL/ordering.cpp)
  ipp=Mt->elements[eid].ipp[0][0];
  nodip_planelt (ipp,intordsm[0][0],ipnums);
  ipp=Mt->elements[eid].ipp[2][2];
  nodip_planelt (ipp,intordsm[2][2],ipnump);
  
  //  node numbers of the element
  Mt->give_elemnodes (eid,nod);
  
  for (i=0;i<nne;i++){
    //  strains at the closest integration point
    Mm->givestrain (lcid,ipnums[i],epss);
    Mm->givestrain (lcid,ipnump[i],epsp);
    
    /*
    eps[0]=epss[0];
    eps[1]=epss[1];
    eps[2]=0.0;
    eps[3]=epsp[0];
    eps[4]=epsp[0];
    eps[5]=epss[0];
    */

    loc_glob_tens_trans (eps,tran);
    
    //  storage of strains to the node
    j=nod[i];
    Mt->nodes[j].storestrain (lcid,0,eps.n,eps);
  }
  destrv (eps);
  
  /*
  allocv (tncompd,eps);
  //  numbers of integration points closest to nodes
  //  (function is from the file GEFEL/ordering.cpp)
  ipp=Mt->elements[eid].ipp[2][2];
  nodip_planelt (ipp,intordsm[2][2],ipnum);
  
  //  node numbers of the element
  Mt->give_elemnodes (eid,nod);
  
  for (i=0;i<nne;i++){
    //  strains at the closest integration point
    Mm->givestrain (lcid,ipnum[i],eps);
    
    loc_glob_tens_trans (eps,tran);
    
    //  storage of strains to the node
    j=nod[i];
    Mt->nodes[j].storestrain (lcid,tncompd,eps.n,eps);
  }
  destrv (eps);
  */
}
  
void shelltr::strains (long lcid,long eid)
{

}

void shelltr::nod_stresses_ip (long lcid,long eid)
{
  
  //  contribution from triangular plane element with rotational degrees of freedom
  Perlt->nod_stresses_ip (lcid,eid,0,0);
    
  //  contribution from triangular plate element
  Dkt->nod_stresses_ip (lcid,eid,2,2);
}


  
void shelltr::stresses (long lcid,long eid)
{
}



void shelltr::inicipval(long eid, long ri, long ci, matrix &nodval, inictype *ictn)
{
  long i, j, k, ipp;
  long ii, jj, nv = nodval.n;
  long nstra;
  double xi, eta;
//  double ipval;
  vector w, gp1, gp2, anv(nne);

  nstra = 0;
  for (j = 0; j < nv; j++) // for all initial values
  {
    for(i = 0; i < nne; i++) // for all nodes on element
      anv[i] = nodval[i][j];
    for (ii = 0; ii < nb; ii++)
    {
      for (jj = 0; jj < nb; jj++)
      {
        ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
        if (intordsm[ii][jj] == 0)
          continue;
        allocv (intordsm[ii][jj],gp1);
        allocv (intordsm[ii][jj],gp2);
        allocv (intordsm[ii][jj],w);
        gauss_points_tr (gp1.a, gp2.a, w.a, intordsm[ii][jj]);
        for (k = 0; k < intordsm[ii][jj]; k++)
        {
          xi=gp1[k];
          eta=gp2[k];
          //  value in integration point
//          ipval = approx_nat (xi,eta,anv);
          if ((ictn[i] & inistrain) && (j < Mm->ip[ipp].ncompstr))
          {
            //Mm->ip[ipp].strain[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & inistress) && (j < nstra + Mm->ip[ipp].ncompstr))
          {
            //Mm->ip[ipp].stress[j] += ipval;
            ipp++;
            continue;
          }
          if ((ictn[i] & iniother) && (j < nv))
          {
            //Mm->ip[ipp].other[j] += ipval;
            ipp++;
            continue;
          }
          ipp++;
        }
        destrv(gp1);  destrv (gp2);  destrv (w);
      }
    }
    if (ictn[i] & inistrain) nstra++;
  }
}
