#include <math.h>
#include "svispl.h"
#include "global.h"
#include "matrix.h"
#include "vector.h"
#include "intpoints.h"


svisplas::svisplas (void)
{
  eta=0.0;
}

svisplas::~svisplas (void)
{

}

void svisplas::read (XFILE *in)
{
  xfscanf (in,"%lf",&eta);
}

double svisplas::gfun (double f)
{
  double g;
  if (f>0.0)  g=f*eta;
  else        g=0.0;
  return g;
}

double svisplas::dergfun (double f)
{
  double dg;
  if (f<0.0)  dg=0.0;
  else        dg=eta;
  return dg;
}

/**
   function computes stresses

   @param ipp - number of integration point
   @param ido - 

   JK, 27.10.2001
*/
void svisplas::nlstresses (long ipp,long ido)
{
  long i,ncomp=Mm->ip[ipp].ncompstr,ncomph=1;
//  double dt;
  vector sig(ncomp),q(ncomph),epsn(ncomp),epso(ncomp),epsp(ncomp);
  matrix d(ncomp,ncomp);

  for (i=0;i<ncomp;i++){
    //  new total strain
    epsn[i] = Mm->ip[ipp].strain[i];
    //  previous total strain
    epso[i] = Mm->ip[ipp].eqother[ido+2*ncomp+i];
    //  increment of irreversible strain
    epsp[i] = Mm->ip[ipp].eqother[ido+ncomp+i];
  }
  
  //  elastic strain
  subv (epsn,epso,epso);
  subv (epso,epsp,epsp);
  //  stiffness matrix of material
  Mm->matstiff(d,ipp);
  //  stress components
  mxv (d,epsp,sig);
  
  //  new data storage
  for (i=0;i<ncomp;i++){
    //Mm->ip[ipp].eqother[ido+i] += sig[i];
    Mm->ip[ipp].eqother[ido+2*ncomp+i] = epsn[i];
    //Mm->ip[ipp].stress[i]=sig[i];
    Mm->ip[ipp].stress[i]+=sig[i];
  }
}

/**
   function computes increments of stress

   @param ipp - number of integration point
   @param ido - 
   
   JK, 29.4.2008
*/
void svisplas::nlstressesincr (long ipp,long ido)
{
  long i,ncomp=Mm->ip[ipp].ncompstr,ncomph=1;
  double dt;
  vector sig(ncomp),q(ncomph),epsn(ncomp),epso(ncomp),epsp(ncomp);
  matrix d(ncomp,ncomp);

  /*********************************/
  //  right hand side computation  //
  /*********************************/
  //  stress component
  for (i=0;i<ncomp;i++){
    //sig[i]=Mm->ip[ipp].eqother[ido+i];
    sig[i]=Mm->ip[ipp].stress[i];
  }
  //  internal variable
  q[0]=Mm->ip[ipp].eqother[ido+3*ncomp];
  
  //dt=Mp->incrvs;
  //dt=Mp->timefun.getval(Mp->time);
  dt=Mp->timecon.actualforwtimeincr ();
  Mm->stiff_deps_vispl (ipp,-1,0,sig,q,dt);
  
  for (i=0;i<ncomp;i++){
    Mm->ip[ipp].eqother[ido+i]=sig[i];
  }
  
}


/**
   function returns tangent stiffness matrix of material
   
   @param ipp - integration point pointer
   @param d - elastic stiffness matrix
   @param ido - index of internal variables for given material in the ipp other array

   12.8.2001
*/
void svisplas::matstiff (matrix &d,long ipp,long ido)
{
  if (Mp->stmat==0){
    //  initial elastic matrix
    Mm->elmatstiff (d,ipp);
  }
  if (Mp->stmat==1){
    //  tangent stiffness matrix
    //  nedodelano
    Mm->elmatstiff (d,ipp);
  }
}


void svisplas::give_interparam (long ipp,long ido,vector &q)
{
  long ncomp;
  ncomp=Mm->ip[ipp].ncompstr;
  
  if (q.n!=1){
    fprintf (stderr,"\n\n uncompatible type of vector of internal parameters in");
    fprintf (stderr,"\n function store_interparam (file %s, line %d).\n",__FILE__,__LINE__);
  }
  
  q[0] = Mm->ip[ipp].eqother[ido+3*ncomp];
}

void svisplas::store_interparam (long ipp,long ido,vector &q)
{
  long ncomp;
  ncomp=Mm->ip[ipp].ncompstr;
  
  if (q.n!=1){
    fprintf (stderr,"\n\n uncompatible type of vector of internal parameters in");
    fprintf (stderr,"\n function store_interparam (file %s, line %d).\n",__FILE__,__LINE__);
  }
  
  Mm->ip[ipp].eqother[ido+3*ncomp]=q[0];
}

/**
   29.4.2008
*/
void svisplas::givestressincr (long ipp,long ido,vector &sig)
{
  long i,ncomp;
  ncomp=Mm->ip[ipp].ncompstr;
  
  if (ncomp!=sig.n){
    print_err("number of components of sig is not equal to ncomp",__FILE__,__LINE__,__func__);
    abort ();
  }

  for (i=0;i<ncomp;i++){
    sig[i]=Mm->ip[ipp].eqother[ido+i];
  }
}

/**
   29.4.2008
*/
void svisplas::storestressincr (long ipp,long ido,vector &sig)
{
  long i,ncomp;
  ncomp=Mm->ip[ipp].ncompstr;
  
  if (ncomp!=sig.n){
    print_err("number of components of sig is not equal to ncomp",__FILE__,__LINE__,__func__);
    abort ();
  }
  
  for (i=0;i<ncomp;i++){
    Mm->ip[ipp].eqother[ido+i]=sig[i];
  }
}
